
import React from 'react';
import { createRoot } from 'react-dom/client';
import { BrowserRouter } from 'react-router-dom';
import App from './App.tsx';
import './index.css';
import { initializeAnalytics } from './lib/analytics';
import { initPerformanceChecks } from './utils/performance-checker';

// Performance optimization: Initialize module loading states
let analyticsLoaded = false;

// Simplified global error handler for runtime errors
window.addEventListener('error', (event) => {
  console.error('Global error caught:', event.error);
  
  // Check if it's a WebSocket-related or unexpected identifier error
  if (event.error && 
     (event.error.toString().includes('WebSocket') || 
      event.error.toString().includes('Unexpected identifier'))) {
    
    console.warn('WebSocket or syntax error detected, attempting recovery...');
    
    // For Vite HMR errors, try to recover by reloading the page
    setTimeout(() => {
      console.warn('Reloading page to recover from HMR error');
      window.location.reload();
    }, 1000);
  }
});

// Main function to start the application
async function startApp() {
  // Initialize analytics (without GTM)
  if (!analyticsLoaded) {
    initializeAnalytics();
    analyticsLoaded = true;
  }
  
  // Initialize the root with error handling
  const rootElement = document.getElementById("root");
  if (!rootElement) {
    console.error("Failed to find root element");
    return;
  }
  
  const root = createRoot(rootElement);
  
  // Reset scroll position to top on initial load
  window.scrollTo(0, 0);
  
  // Add a timing mark for performance analysis
  if (performance && 'mark' in performance) {
    performance.mark('react-mount-start');
  }
  
  // Initialize performance checks in development mode
  if (process.env.NODE_ENV === 'development') {
    initPerformanceChecks();
  }
  
  // Render the app with error boundary at the top level
  // Wrap the App component with BrowserRouter
  try {
    root.render(
      <React.StrictMode>
        <BrowserRouter>
          <App />
        </BrowserRouter>
      </React.StrictMode>
    );
  } catch (error) {
    console.error('Error rendering the application:', error);
    rootElement.innerHTML = `
      <div style="color: red; padding: 20px; text-align: center;">
        <h1>Application Error</h1>
        <p>Sorry, something went wrong. Please try refreshing the page.</p>
        <pre>${error instanceof Error ? error.message : String(error)}</pre>
      </div>
    `;
  }
  
  // Mark render time for performance metrics
  if (performance && 'mark' in performance && 'measure' in performance) {
    performance.mark('react-mount-end');
    performance.measure('react-mount', 'react-mount-start', 'react-mount-end');
  }
  
  // Lazy load other non-critical modules after page is interactive
  if ('requestIdleCallback' in window) {
    (window as any).requestIdleCallback(loadNonCritical, { timeout: 2000 });
  } else {
    // Fallback for browsers without requestIdleCallback
    setTimeout(loadNonCritical, 2000);
  }
}

// Function to lazily load non-critical modules
function loadNonCritical() {
  // Detect if user is on a mobile device
  const isMobileDevice = /Android|webOS|iPhone|iPad|iPod|BlackBerry|IEMobile|Opera Mini/i.test(navigator.userAgent);
  
  // Preload fewer routes on mobile to save bandwidth
  if ('IntersectionObserver' in window) {
    // On mobile, only preload the most critical routes
    const prefetchRoutes = isMobileDevice ? 
      ['/contact', '/conventional-loans'] : 
      ['/conventional-loans', '/fha-loans', '/jumbo-loans', '/va-loans'];
    
    // Detect viewport visibility and prefetch when idle
    const observer = new IntersectionObserver(
      (entries) => {
        if (entries[0].isIntersecting && 'requestIdleCallback' in window) {
          (window as any).requestIdleCallback(() => {
            prefetchRoutes.forEach(route => {
              const link = document.createElement('link');
              link.rel = 'prefetch';
              link.href = route;
              document.head.appendChild(link);
            });
          }, { timeout: isMobileDevice ? 10000 : 5000 });
          
          observer.disconnect();
        }
      },
      { rootMargin: isMobileDevice ? '100px 0px' : '200px 0px' }
    );
    
    // Start observing the footer as a trigger point
    setTimeout(() => {
      const footer = document.querySelector('footer');
      if (footer) observer.observe(footer);
    }, isMobileDevice ? 5000 : 3000);
  }
}

// Start the application with better error handling
startApp().catch(err => {
  console.error('Error starting the application:', err);
  // Display a fallback error message
  const rootElement = document.getElementById("root");
  if (rootElement) {
    rootElement.innerHTML = `
      <div style="color: red; padding: 20px; text-align: center;">
        <h1>Failed to load the application</h1>
        <p>Please refresh the page or try again later.</p>
        <pre>${err instanceof Error ? err.message : String(err)}</pre>
      </div>
    `;
  }
});
